#include <iostream>
#include <string>

// hold
//
// ... by Eli Fulkerson.  See http://www.elifulkerson.com for updates
//
// hold is intended to buffer pipe input, holding stdin until it ceases then dumping the
// whole bucket to stdout at once.
//
// This was written because I was having trouble using Windows based command line utilities
// under cygwin.  In particular, if I was trying to parse the output of "tracert" via a
// second program using Console.Readline() the second program's input would give up and die
// as soon as tracert delayed.
//
// An extremely cursory and shallow bit of research later, apparently there is a difference
// between how Cygwin and Windows buffer their input, in particular (although I haven't
// verified this) null writes being sent on stdout.  Under Cygwin the programs talk to
// eachother directly, the null write is interpreted as EOF, and BOOM.  Under Windows,
// apparently, there is some sort of other layer that appears between them and causes this
// to not happen.
//
// Reference:
// https://cygwin.com/ml/cygwin/2012-04/msg00613.html
//
// ... that being said, I really only did a cursory skim.  That discussion appears to be similar
// but not identical to the difficulties I had, seemed plausible and seemed unlikely to result in
// a fix.

/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


using namespace std;

int main(int argc, char* argv[]) {

	string input = "";
	string output = "";

	bool mode_verbose = true;

	//parse some arguments
	for ( int i = 0; i < argc; i++) {

		std::string arg = argv[i];
		
		if ( arg == "-q") {
			mode_verbose = false;
		}

		if (arg == "-v" || arg == "--version") {
			cout << "hold v0.1  17 October 2015" << endl
				 << "by Eli Fulkerson.  See http://www.elifulkerson.com for updates" << endl << flush;
			return 0;
		}


		if (arg == "-h" || arg == "--help" || arg == "--usage") {
			cout << "Syntax: your_command |hold | your_other_command" << endl
				 << "Options:" << endl
				 << "-q     : Turn off the 'delaying' and 'dumping' messages" << endl
				 << "-v     : Display version information" << endl << flush;
			return 0;
		}
	}
	
	while (true) {
		getline(cin, input);

		if (mode_verbose) {
			cerr << "\r" << "hold: delaying " << output.size() << " bytes." << flush;
		}
		
		output.append(input);
		output.append("\n");

		if (!cin) {
			if (mode_verbose) {
				cerr << "\r" << "hold: dumping " << output.size() << " bytes.  " << flush;
			}
			cout << output;
			return 0;
		}
	}

	return 0;
}
